<?php

namespace Drupal\transform_api\Plugin\Transform\Type;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\transform_api\Configs\EntityTransformDisplayInterface;
use Drupal\transform_api\Entity\EntityTransformDisplay;
use Drupal\transform_api\EventSubscriber\TransformationCache;
use Drupal\transform_api\FieldTransformManager;
use Drupal\transform_api\Repository\EntityTransformRepositoryInterface;
use Drupal\transform_api\Transform\EntityTransform;
use Drupal\transform_api\Transform\FieldTransform;
use Drupal\transform_api\Transform\SimpleTransform;
use Drupal\transform_api\Transform\TransformInterface;
use Drupal\transform_api\TransformationTypeBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @TransformationType(
 *  id = "entity",
 *  title = "Entity transform"
 * )
 */
class Entity extends TransformationTypeBase {

  /**
   * @var \Drupal\transform_api\EventSubscriber\TransformationCache
   */
  protected TransformationCache $transformationCache;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * @var \Drupal\transform_api\FieldTransformManager
   */
  protected FieldTransformManager $fieldTransformManager;

  /**
   * @var \Drupal\transform_api\Repository\EntityTransformRepositoryInterface
   */
  protected EntityTransformRepositoryInterface $entityTransformRepository;

  /**
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entityTypeManager, TransformationCache $transformationCache, FieldTransformManager $fieldTransformManager, EntityTransformRepositoryInterface $entityTransformRepository, ModuleHandlerInterface $moduleHandler) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entityTypeManager;
    $this->transformationCache = $transformationCache;
    $this->fieldTransformManager = $fieldTransformManager;
    $this->entityTransformRepository = $entityTransformRepository;
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * @inheritDoc
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($configuration, $plugin_id, $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('transform_api.transformation_cache'),
      $container->get('plugin.manager.transform_api.field_transform'),
      $container->get('transform_api.entity_display.repository'),
      $container->get('module_handler')
    );
  }

  /**
   * @inheritDoc
   */
  public function transform(TransformInterface $transform) {
    /** @var EntityTransform $entity_transform */
    $entity_transform = $transform;
    if (is_array($transform->getValue('ids'))) {
      return $this->prepareMultipleTransforms($transform->getValue('entity_type'), $transform->getValue('ids'), $transform->getValue('transform_mode') ?? EntityTransformRepositoryInterface::DEFAULT_DISPLAY_MODE);
    } elseif (!is_null($entity_transform->getEntity())) {
      return $this->transformEntity($entity_transform->getEntity(), $entity_transform->getFields(), $entity_transform->getDisplay());
    } else {
      return $this->loadAndTransformEntity($transform->getValue('entity_type'), $transform->getValue('ids'), $transform->getValue('transform_mode') ?? EntityTransformRepositoryInterface::DEFAULT_DISPLAY_MODE);
    }
  }

  public function loadAndTransformEntity($entity_type_id, $id, $transform_mode = EntityTransformRepositoryInterface::DEFAULT_DISPLAY_MODE, $langcode = '') {
    $storage = $this->entityTypeManager->getStorage($entity_type_id);
    /** @var FieldableEntityInterface $entity */
    $entity = $storage->load($id);
    if (!empty($langcode)) {
      $entity = $entity->getTranslation($langcode);
    }
    if (!empty($entity)) {
      return $this->prepareAndTransformEntity($entity, $transform_mode);
    } else {
      return [];
    }
  }

  public function prepareMultipleTransforms($entity_type_id, array $ids, $transform_mode = EntityTransformRepositoryInterface::DEFAULT_DISPLAY_MODE, $langcode = '') {
    $transformation = [];
    $cached = [];
    $missing_ids = [];
    foreach ($ids as $id) {
      $transform = new EntityTransform($entity_type_id, $id, $transform_mode);
      $entity_transformation = $this->transformationCache->get($transform);
      if ($entity_transformation === FALSE) {
        $missing_ids[] = $id;
      } else {
        $cached[$id] = $entity_transformation;
      }
    }

    if (!empty($missing_ids)) {
      $storage = $this->entityTypeManager->getStorage($entity_type_id);
      /** @var FieldableEntityInterface[] $entities */
      $entities = $storage->loadMultiple($missing_ids);
      if (!empty($entities)) {
        $displays = EntityTransformDisplay::collectTransformDisplays($entities, $transform_mode);
        $fields = [];
        $field_values = [];
        foreach ($missing_ids as $id) {
          if (empty($entities[$id])) {
            continue;
          }
          $entity = $entities[$id];
          if (!empty($langcode)) {
            $entities[$id] = $entity = $entity->getTranslation($langcode);
          }

          $bundle = $entity->bundle();
          if (empty($fields[$bundle])) {
            $fields[$bundle] = [];
            $field_values[$bundle] = [];
            foreach ($displays[$bundle]->getComponents() as $field_name => $definition) {
              $configuration = [
                'field_definition' => $entity->get($field_name)
                  ->getFieldDefinition(),
                'settings' => $definition['settings'] ?? [],
                'label' => $definition['label'] ?? '',
                'transform_mode' => $transform_mode,
                'third_party_settings' => $definition['third_party_settings'] ?? []
              ];
              $fields[$bundle][$field_name] = $this->fieldTransformManager->createInstance($definition['type'], $configuration);
              $field_values[$bundle][$field_name] = [];
            }
          }
          foreach ($fields[$bundle] as $field_name => $plugin) {
            $field_values[$bundle][$field_name][$id] = $entity->get($field_name);
          }
        }
        foreach ($fields as $bundle => $plugins) {
          foreach ($plugins as $field_name => $plugin) {
            $plugin->prepareTransform($field_values[$bundle][$field_name]);
          }
        }
      }
    }

    foreach ($ids as $id) {
      if (in_array($id, $missing_ids)) {
        if (empty($entities[$id])) {
          continue;
        }
        $transformation[] = EntityTransform::createPrepared($entities[$id], $fields[$entities[$id]->bundle()], $displays[$entities[$id]->bundle()]);
      } else {
        $transformation[] = new SimpleTransform($cached[$id]);
      }
    }

    return $transformation;
  }

  public function prepareAndTransformEntity(FieldableEntityInterface $entity, $transform_mode = EntityTransformRepositoryInterface::DEFAULT_DISPLAY_MODE) {
    $plugins = [];
    $display = EntityTransformDisplay::collectTransformDisplay($entity, $transform_mode);
    foreach ($display->getComponents() as $field_name => $definition) {
      $configuration = [
        'field_definition' => $entity->get($field_name)->getFieldDefinition(),
        'settings' => $definition['settings'] ?? [],
        'label' => $definition['label'] ?? '',
        'transform_mode' => $transform_mode ?? EntityTransformRepositoryInterface::DEFAULT_DISPLAY_MODE,
        'third_party_settings' => $definition['third_party_settings'] ?? []
      ];
      $plugins[$field_name] = $this->fieldTransformManager->createInstance($definition['type'], $configuration);
      $plugins[$field_name]->prepareTransform([$entity->id() => $entity->get($field_name)]);
    }
    return $this->transformEntity($entity, $plugins, $display);
  }

  public function transformEntity(EntityInterface $entity, array $plugins = [], EntityTransformDisplayInterface $display = NULL) {
    $cacheMetadata = CacheableMetadata::createFromObject($entity);
    $transformation = [];
    $transformation['type'] = 'entity';
    $transformation['entity_type'] = $entity->getEntityTypeId();
    $transformation['bundle'] = $entity->bundle();
    $transformation['id'] = $entity->id();
    $transformation['label'] = $entity->label();
    $transformation['transform_mode'] = $display->getMode();
    $transformation['langcode'] = $entity->language()->getId();
    $transformation['#entity'] = $entity;
    if ($entity instanceof FieldableEntityInterface && !empty($display)) {
      $cacheMetadata->addCacheableDependency($display);
      foreach ($display->getComponents() as $field_name => $definition) {
        $transformation[$definition['name'] ?? $field_name] = new FieldTransform($plugins[$field_name], $entity, $field_name);
      }
    }
    $cacheMetadata->applyTo($transformation);
    return $transformation;
  }
}
